BuffTheTree = LibStub("AceAddon-3.0"):NewAddon("BuffTheTree", "AceEvent-3.0", "AceConsole-3.0")
local grat = LibStub("LibGratuity-3.0")
local BuffTheTree = BuffTheTree
local defaults = { profile = {
	locked = false,
	blessings = {},
	incombat = false,
	x = 200,
	y = 200,
	anchor = "BOTTOMLEFT",
	parent = UIParent,
	point = UIParent,
	debug = false,
	config = false,
	showinbgs = false,
	alwaystrue = true,
	imbue = false,
}}

function BuffTheTree:DebugMessage(msg, level)
	if not self.opt.debug then return end
	if not level then level = 2 end
	if level <= self.opt.debug then
		msg = "|cff7cfc00BuffTheTree debugging:|r "..msg;
		print(msg)
	end
end

function BuffTheTree:UpdateTimers()
	for i, k in pairs(BuffTheTree) do
		if i:find("buff%d") then
			if k.name then
				local expires = (select(7, UnitBuff("player", k.name)))
				if expires then
					local totalminutes = (expires-GetTime())/60
					local minutes = math.floor(totalminutes)
					local seconds = math.floor((totalminutes-minutes)*60)
					if seconds < 10 then seconds = "0"..seconds end
					local text
					if minutes > 0 then
						text = minutes..":"..seconds
					else
						text = tostring(seconds)
					end
					if k.textcache ~= text then
						k.textcache = text
						k.text:SetText(text)
					end
				end
			end
			if k.slot then
				local duration
				if k.slot == GetInventorySlotInfo("MainHandSlot") then
					duration = (select(2, GetWeaponEnchantInfo()))
				elseif k.slot == GetInventorySlotInfo("SecondaryHandSlot") then
					duration = (select(5, GetWeaponEnchantInfo()))
				else
					return
				end
				local text = BuffTheTree:GetWeaponImbueDuration(duration)
				if text ~= k.textcache then
					k.textcache = text
					k.text:SetText(text)
				end
			end
		end
	end
end

function BuffTheTree:IsBuffEligible(class, num)
	if not num then self:DebugMessage("Eligibility check cancelled due to no required number being passed"); return false end
	if self.opt.config then self:DebugMessage("Config mode: Eligible"); return true end
	if self.opt.alwaystrue and not ((GetNumRaidMembers() > 0 and self.opt.showinbgs) or GetRealNumRaidMembers() > 0) then self:DebugMessage("Always true: Eligible"); return true end
	local numinraid = 0
	for i=1, GetNumRaidMembers() do
		if (select(2, UnitClass("raid"..i))) == class then
			numinraid = numinraid+1;
			if numinraid >= num then
				self:DebugMessage("There are more than "..num.." of "..class.." in the raid");
				return true
			end
		end
	end
	self:DebugMessage("There are not enough of "..class.." in the raid. (Found "..numinraid..", need "..num..")")
	return false
end

function BuffTheTree:GetSpecialClassFilter(filter)
	self:DebugMessage("Attempting to retrieve filter: "..filter);
	if filter == "mana" then
		local filtertable = {
			["WARRIOR"] = false,
			["ROGUE"] = false,
			["SHAMAN"] = true,
			["DRUID"] = true,
			["PALADIN"] = true,
			["HUNTER"] = true,
			["PRIEST"] = true,
			["WARLOCK"] = true,
			["DEATHKNIGHT"] = false,
			["MAGE"] = true,
		}
		self:DebugMessage("Successfully returned mana filter table");
		return filtertable
	end
	if filter == "all" then
		local filtertable = {
			["WARRIOR"] = true,
			["ROGUE"] = true,
			["SHAMAN"] = true,
			["DRUID"] = true,
			["PALADIN"] = true,
			["HUNTER"] = true,
			["PRIEST"] = true,
			["WARLOCK"] = true,
			["DEATHKNIGHT"] = true,
			["MAGE"] = true,
		}
		self:DebugMessage("Successfully returned all filter table");
		return filtertable
	end
end

function BuffTheTree:GetBlessingPrio(qblessing)
	for prio, blessing in pairs(self.opt.blessings) do
		if blessing == qblessing then
			self:DebugMessage("Blessing of "..qblessing.." is priority "..prio..". Returning that value...");
			return prio
		end
	end
	self:DebugMessage("Blessing of "..qblessing.." doesn't have a priority.");
end

function BuffTheTree:GetOptions()
	local opt = {
		display = {
			name = "Display settings",
			type = "group",
			order = 1,
			args = {
				locked = {
					name = "Locked",
					type = "toggle",
					get = function() return self.opt.locked end,
					set = function(info, value) self.opt.locked = value end,
					order = 1,
				},
				incombat = {
					name = "Show in combat",
					type = "toggle",
					get = function() return self.opt.incombat end,
					set = function(info, value) self.opt.incombat = value end,
					order = 2,
				},
				debug = {
					name = "Debug",
					type = "range",
					min = 0,
					max = 2,
					step = 1,
					get = function() return self.opt.debug or 0 end,
					set = function(info, value) self.opt.debug = value end,
					order = 3,
				},
				config = {
					name = "Config mode",
					type = "toggle",
					get = function() return self.opt.config end,
					set = function(info, value) self.opt.config = value self:UNIT_AURA() end,
					order = 4,
				},
				showinbg = {
					name = "Show in battlegrounds",
					type = "toggle",
					get = function() return self.opt.showinbgs end,
					set = function(info, value) self.opt.showinbgs = value self:UNIT_AURA() end,
					order = 5,
				},
				alwaystrue = {
					name = "Always show applied",
					type = "toggle",
					get = function() return self.opt.alwaystrue end,
					set = function(info, value) self.opt.alwaystrue = value self:UNIT_AURA() end,
					order = 6,
				},
				imbue = {
					name = "Show Weapon imbues",
					type = "toggle",
					get = function() return self.opt.imbue end,
					set = function(info, value) self.opt.imbue = value self:UNIT_AURA() end,
					order = 7,
				},
			},
		},
		buffsettings = {
			name = "Buff settings",
			type = "group",
			order = 2,
			args = {
				blessings = {
					type = "group",
					name = "Blessings",
					dialogInline = true,
					order = 1,
					args = {
						first = {
							name = "First blessing",
							type = "select",
							get = function() return self.opt.blessings[1] or "none" end,
							set = function(info, value) if value == "none" then self.opt.blessings[1] = nil; self:UNIT_AURA() end for i=1, 4 do if i~=1 then if self.opt.blessings[i] == value then self.opt.blessings[i] = self.opt.blessings[1]; end end end self.opt.blessings[1] = value; self:UNIT_AURA() end,
							order = 1,
							values = {["kings"] = "Blessing of Kings", ["wisdom"] = "Blessing of Wisdom", ["might"] = "Blessing of Might", ["sanc"] = "Blessing of Sanctuary", ["none"] = "None"},
						},
						second = {
							name = "Second blessing",
							type = "select",
							get = function() return self.opt.blessings[2] or "none" end,
							set = function(info, value) if value == "none" then self.opt.blessings[2] = nil; self:UNIT_AURA() end for i=1, 4 do if i~=2 then if self.opt.blessings[i] == value then self.opt.blessings[i] = self.opt.blessings[2]; end end end self.opt.blessings[2] = value; self:UNIT_AURA() end,
							order = 2,
							values = {["kings"] = "Blessing of Kings", ["wisdom"] = "Blessing of Wisdom", ["might"] = "Blessing of Might", ["sanc"] = "Blessing of Sanctuary", ["none"] = "None"},
						},
						third = {
							name = "Third blessing",
							type = "select",
							get = function() return self.opt.blessings[3] or "none" end,
							set = function(info, value) if value == "none" then self.opt.blessings[3] = nil; self:UNIT_AURA() end for i=1, 4 do if i~=3 then if self.opt.blessings[i] == value then self.opt.blessings[i] = self.opt.blessings[3]; end end end self.opt.blessings[3] = value; self:UNIT_AURA() end,
							order = 3,
							values = {["kings"] = "Blessing of Kings", ["wisdom"] = "Blessing of Wisdom", ["might"] = "Blessing of Might", ["sanc"] = "Blessing of Sanctuary", ["none"] = "None"},
						},
						fourth = {
							name = "Fourth blessing",
							type = "select",
							get = function() return self.opt.blessings[4] or "none" end,
							set = function(info, value) if value == "none" then self.opt.blessings[4] = nil; self:UNIT_AURA() end for i=1, 4 do if i~=4 then if self.opt.blessings[i] == value then self.opt.blessings[i] = self.opt.blessings[4]; end end end self.opt.blessings[4] = value; self:UNIT_AURA() end,
							order = 4,
							values = {["kings"] = "Blessing of Kings", ["wisdom"] = "Blessing of Wisdom", ["might"] = "Blessing of Might", ["sanc"] = "Blessing of Sanctuary", ["none"] = "None"},
						},
					},
				},
			},
		},
	}
	return opt
end

function BuffTheTree:OnProfileEnable()
	self.opt = self.db.profile
end

function BuffTheTree:GetBuffTable()
	self:DebugMessage("Retrieving buff table");
	local buffs = {
		pwf = {
			buffinfo = {
				[48162] = {GetSpellInfo(48162), "PRIEST", 1},
				[48161] = {GetSpellInfo(48161), "PRIEST", 1},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_holy_wordfortitude",
		},
		motw = {
			buffinfo = {
				[48469] = {GetSpellInfo(48469), "DRUID", 1},
				[48470] = {GetSpellInfo(48470), "DRUID", 1},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_nature_giftofthewild",
		},
		kings = {
			buffinfo = {
				[20217] = {GetSpellInfo(20217), "PALADIN", self:GetBlessingPrio("kings")},
				[25898] = {GetSpellInfo(25898), "PALADIN", self:GetBlessingPrio("kings")},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_magic_magearmor",
		},
		wisdom = {
			buffinfo = {
				[48936] = {GetSpellInfo(48936), "PALADIN", self:GetBlessingPrio("wisdom")},
				[48938] = {GetSpellInfo(48938), "PALADIN", self:GetBlessingPrio("wisdom")},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_holy_sealofwisdom",
		},
		might = {
			buffinfo = {
				[48932] = {GetSpellInfo(48932), "PALADIN", self:GetBlessingPrio("might")},
				[48934] = {GetSpellInfo(48934), "PALADIN", self:GetBlessingPrio("might")},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_holy_fistofjustice",
		},
		sanctuary = {
			buffinfo = {
				[20911] = {GetSpellInfo(20911), "PALADIN", self:GetBlessingPrio("sanc")},
				[25899] = {GetSpellInfo(25899), "PALADIN", self:GetBlessingPrio("sanc")},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_holy_greaterblessingofsanctuary",
		},
		ai = {
			buffinfo = {
				[61316] = {GetSpellInfo(61316), "MAGE", 1},
				[43002] = {GetSpellInfo(43002), "MAGE", 1},
				[61024] = {GetSpellInfo(61024), "MAGE", 1},
				[42995] = {GetSpellInfo(42995), "MAGE", 1},
			},
			classfilter = self:GetSpecialClassFilter("mana"),
			texture = "interface/icons/spell_holy_arcaneintellect",
		},
		spirit = {
			buffinfo = {
				[48073] = {GetSpellInfo(48073), "PRIEST", 1},
				[48074] = {GetSpellInfo(48074), "PRIEST", 1},
			},
			classfilter = self:GetSpecialClassFilter("mana"),
			texture = "interface/icons/spell_holy_prayerofspirit",
		},
		sprot = {
			buffinfo = {
				[48170] = {GetSpellInfo(48170), "PRIEST", 1},
				[48169] = {GetSpellInfo(48169), "PRIEST", 1},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_holy_prayerofshadowprotection",
		},
		flask = {
			buffinfo = {
				[54212] = {GetSpellInfo(54212), (select(2, UnitClass("player"))), 1},
				[53755] = {GetSpellInfo(53755), (select(2, UnitClass("player"))), 1},
				[53758] = {GetSpellInfo(53758), (select(2, UnitClass("player"))), 1},
				[53760] = {GetSpellInfo(53760), (select(2, UnitClass("player"))), 1},
				[67019] = {GetSpellInfo(67019), (select(2, UnitClass("player"))), 1},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/inv_alchemy_endlessflask_05",
		},
		bufffood = {
			buffinfo = {
				["food"] = {GetSpellInfo(57294), (select(2, UnitClass("player"))), 1},
			},
			classfilter = self:GetSpecialClassFilter("all"),
			texture = "interface/icons/spell_misc_food",
		},
	}
	self:DebugMessage("Returning buff table successful");
	return buffs
end

function BuffTheTree:StartMoving()
	local self = BuffTheTree
	if self.opt.locked then self:DebugMessage("Frames are locked, not moving"); return end
	self:DebugMessage("Moving frames now");
	self.buff1:StartMoving()
end

function BuffTheTree:StopMoving()
	local self = BuffTheTree
	self:DebugMessage("Frames aren't dragged anymore, save position")
	self.buff1:StopMovingOrSizing()
	self.opt.anchor, self.opt.parent, self.opt.point, self.opt.x, self.opt.y = self.buff1:GetPoint()
	if not self.opt.parent then self.opt.parent = UIParent end
	self.opt.parent = self.opt.parent:GetName()
	self:DebugMessage("Position saved");
end

function BuffTheTree:CreateFrame(i)
	self:DebugMessage("Attempting to create frame "..i);
	local f = CreateFrame("Button", "BuffTheTreeBuff"..i, UIParent)
	f:SetWidth(100)
	f:SetHeight(35)
	if i == 1 then
		self:DebugMessage("Frame 1 is being anchored to UIParent. X: "..self.opt.x.." Y: "..self.opt.y, 1)
		f:SetScript("OnUpdate", BuffTheTree.UpdateTimers)
		if _G[self.opt.parent] then
			f:SetPoint(self.opt.anchor, _G["self.opt.parent"], self.opt.point, self.opt.x, self.opt.y)
		else
			f:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", ((UIParent:GetWidth()/2)-50), ((UIParent:GetHeight()/2)-17.5))
		end
		f:SetMovable(true)
	else
		f:SetPoint("TOPLEFT", self["buff"..(i-1)], "BOTTOMLEFT", 0, 0)
	end
	f:SetScript("OnEnter", function(self)
		if not (self.slot or self.name) then return end
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:ClearAllPoints()
		--GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
		if self.name then
			GameTooltip:ClearLines()
			local c = 1
			while true do
				if UnitBuff("player", c) == self.name then
					break;
				end
				if not UnitBuff("player", c) then
					c = nil;
					break;
				end
				c = c+1;
			end
			GameTooltip:SetUnitBuff("player", c)
		end
		if self.slot then
			GameTooltip:ClearLines()
			GameTooltip:SetInventoryItem("player", self.slot)
		end
		GameTooltip:Show()
	end)
	f:SetScript("OnLeave", function() GameTooltip:Hide(); end)
	f:RegisterForClicks("AnyUp")
	f:SetScript("OnMouseDown", BuffTheTree.StartMoving)
	f:SetScript("OnMouseUp", BuffTheTree.StopMoving)
	self["buff"..i] = f;
	self:DebugMessage("Trunk frame created");
	self:DebugMessage("Creating Icon");
	local f2 = CreateFrame("Frame", "BuffTheTreeBuff"..i.."Icon", f)
	f2:SetWidth(35)
	f2:SetHeight(35)
	f2:SetPoint("TOPLEFT", f, 0, 0)
	f.icon = f2;
	self:DebugMessage("Icon created");
	self:DebugMessage("Creating fontstring");
	local f3 = f2:CreateFontString(nil, "ARTWORK")
	f3:SetFontObject(GameFontHighlightSmall)
	f3:SetHeight(35)
	f3:SetWidth(65)
	f3:ClearAllPoints()
	f3:SetPoint("RIGHT", f, "RIGHT", 0, 0)
	f3:SetJustifyH("LEFT");
	f.text = f3;
	self:DebugMessage("Fontstring created");
	self:DebugMessage("Creating fontstring background")
	local f4 = CreateFrame("Frame", nil, f2)
	f4:SetWidth(20)
	f4:SetHeight(12)
	f4:ClearAllPoints()
	f4:SetPoint("CENTER", f2, "CENTER", 0, 0)
	f4:SetBackdrop({bgFile = "Interface\\Addons\\BuffTheTree\\blank", edgeFile = false, tile = true, tileSize = 10, edgeSize = 0, insets = { left = 0, right = 0, top = 0, bottom = 0}})
	f4:SetBackdropColor(0, 0, 0)	
	f.imbuebg = f4
	self:DebugMessage("Fontstring background created")
	self:DebugMessage("Creating secondary fontstring")
	local f5 = f4:CreateFontString(nil, "ARTWORK")
	f5:SetFontObject(GameFontHighlightSmall)
	f5:SetTextColor(1, 1, 1)
	f5:SetHeight(35)
	f5:SetWidth(35)
	f5:ClearAllPoints()
	f5:SetPoint("CENTER", f4, "CENTER", 0, 0.8)
	f5:SetJustifyH("CENTER")
	f.imbue = f5
	self:DebugMessage("Secondary fontstring created")
	self:DebugMessage("Done creating frame "..i, 1)
	f:Show()
	f2:Show()
	f3:Show()
	f4:Show()
	f5:Show()
end

function BuffTheTree:IsInRaid()
	if self.opt.showinbgs and (GetNumRaidMembers() > 0) then
		return true
	end
	if (GetRealNumRaidMembers() > 0) then
		return true
	end
	if self.opt.config then
		return true
	end
	if self.opt.alwaystrue then
		return true
	end
	return false
end

function BuffTheTree:ShouldShowMissing()
	if self.opt.showinbgs and (GetNumRaidMembers() > 0) then
		return true
	end
	if (GetRealNumRaidMembers() > 0) then
		return true
	end
	if self.opt.config then
		return true
	end
	return false
end

function BuffTheTree:ShouldOverrideRequired()
	if self.opt.config then
		return true
	end
	if self.opt.alwaystrue then
		return true
	end
	return false
end

function BuffTheTree:ShouldShowWeaponImbues()
	if not self.opt.imbue then
		return false
	end
	local class = (select(2, UnitClass("player")))
	if class == "ROGUE" or class == "SHAMAN" then
		return true
	end
	return false
end

function BuffTheTree:GetWeaponImbueDuration(duration)
	local totalminutes = duration/60000
	local minutes = math.floor(totalminutes)
	local seconds = math.floor((totalminutes-minutes)*60)
	if seconds < 10 then seconds = "0"..seconds end
	if minutes < 10 then minutes = "0"..minutes end
	local text = minutes..":"..seconds
	return text
end

function BuffTheTree:GetShamanTexture(imbue)
	local name, texture
	local imbues = {58804, 51994, 58790, 10399, 58796}
	for _, k in pairs(imbues) do
		name, _, texture = GetSpellInfo(k)
		if string.match(name, imbue) then break; end
	end
	if name then
		return texture
	end
end

function BuffTheTree:GetWeaponImbueNameTexture(slot)
	grat:SetInventoryItem("player", slot)
	local texture, name
	_, _, name = grat:Find("^(.+) %(%d+ [^%)]+%)$")
	if not name then return end
	if (select(2, UnitClass("player"))) == "ROGUE" then
		texture = (select(10, GetItemInfo(name)))
		if not texture then return end
		return name, texture
	elseif (select(2, UnitClass("player"))) == "SHAMAN" then
		name = strmatch(name, "%S+")
		texture = self:GetShamanTexture(name)
		if name and texture then
			return name, texture
		end
	end
end

function BuffTheTree:GetMissingTexture(class, offhand)
	local tex
	if class == "ROGUE" then
		tex = "interface/icons/ability_poisons"
	elseif class == "SHAMAN" then
		tex = GetInventoryItemTexture("player", GetInventorySlotInfo(offhand and "SecondaryHandSlot" or "MainHandSlot"))
	end
	if tex then return tex end
end

function BuffTheTree:UNIT_AURA(event, unit)
	if unit ~= "player" and (event == "UNIT_AURA" or event == "UNIT_INVENTORY_CHANGED") then self:DebugMessage("Unit isn't player"); return end
	for c, k in pairs(self) do
		if c:find("buff%d") then k:Hide() end
	end
	if not self:IsInRaid() then return end
	local buffs = self:GetBuffTable()
	local i = 1
	for _, data in pairs(buffs) do
		if data.classfilter[(select(2, UnitClass("player")))] then
			local found, elig
			for id, info in pairs(data.buffinfo) do
				if self:IsBuffEligible(info[2], info[3]) or self:ShouldOverrideRequired() then
					if self:IsBuffEligible(info[2], info[3]) then
						elig = true
					end
					local texture, expires
					--self:DebugMessage("Retrieving buff with name "..info[1])
					_, _, texture, _, _, _, expires = UnitBuff("player", info[1])
					if texture and expires then
						if not self["buff"..i] then
							self:CreateFrame(i)
						end
						local f = self["buff"..i]
						self:DebugMessage(info[1].." found.");
						self:DebugMessage("Using "..f:GetName().." for "..info[1], 1)
						f.name = info[1]
						f.slot = nil
						self:DebugMessage("Texture is "..texture);
						f.icon:SetBackdrop({bgFile = texture, edgeFile = false, tile = true, tileSize = 31, edgeSize = 0, insets = { left=0, right=0, top=0, bottom=0}});
						f.icon:SetBackdropColor(1, 1, 1)
						local totalminutes = (expires-GetTime())/60
						local minutes = math.floor(totalminutes)
						local seconds = math.floor((totalminutes-minutes)*60)
						if seconds < 10 then seconds = "0"..seconds end
						if minutes < 10 then minutes = "0"..minutes end
						local text = minutes..":"..seconds
						self:DebugMessage("Text is "..text)
						f.text:SetText(text)
						f.text:SetTextColor(1, 1, 1)
						f.imbue:SetText(nil)
						f.imbuebg:Hide()
						f:Show()
						found = true
						--self:DebugMessage(info[1].." set to displayed");
						i = i+1;
						break;
					end
				end
			end
			if (not found and elig) and self:ShouldShowMissing() then
				if not self["buff"..i] then
					self:CreateFrame(i);
				end
				local f = self["buff"..i]
				self:DebugMessage("Buff with texture "..data.texture.." not found.");
				self:DebugMessage("Using "..f:GetName().." for missing buff with texture "..data.texture, 1)
				f.name = nil
				f.slot = nil
				f.icon:SetBackdrop({bgFile = data.texture, edgeFile = false, tile = true, tileSize = 31, edgeSize = 0, insets = { left = 0, right = 0, top = 0, bottom = 0}});
				f.icon:SetBackdropColor(0.6, 0, 0)
				f.text:SetText("MISSING")
				f.text:SetTextColor(0.6, 0, 0)
				f.imbue:SetText(nil)
				f.imbuebg:Hide()
				f:Show()
				i=i+1
				--self:DebugMessage("Successfully set to MISSING");
			end
		end
	end
	if not self:ShouldShowWeaponImbues() then return end
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo()
	if hasMainHandEnchant and mainHandCharges == 0 then
		local name, texture = self:GetWeaponImbueNameTexture(GetInventorySlotInfo("MainHandSlot"))
		local text = self:GetWeaponImbueDuration(mainHandExpiration)
		if not self["buff"..i] then
			self:CreateFrame(i)
		end
		local f = self["buff"..i]
		self:DebugMessage("Weapon imbue: "..name, 2)
		self:DebugMessage("Using "..f:GetName().." for "..name, 1)
		f.slot = GetInventorySlotInfo("MainHandSlot")
		f.name = nil
		f.icon:SetBackdrop({bgFile = texture, edgeFile = false, tile=true, tileSize = 31, edgeSize = 0, insets = { left=0, right=0, top=0, bottom=0}})
		f.icon:SetBackdropColor(1, 1, 1)
		f.text:SetText(text)
		f.text:SetTextColor(1, 1, 1)
		f.imbue:SetText("MH")
		f.imbuebg:Show()
		f:Show()
		i=i+1;
	else
		local texture = self:GetMissingTexture((select(2, UnitClass("player"))))
		self:DebugMessage("Using "..f:GetName().." for missing buff with texture "..texture, 1)
		if not self["buff"..i] then
			self:CreateFrame(i)
		end
		local f = self["buff"..i]
		f.name = nil
		f.slot = nil
		f.icon:SetBackdrop({bgFile = texture, edgeFile = false, tile = true, tileSize = 31, edgeSize = 0, insets = { left = 0, right = 0, top = 0, bottom = 0}})
		f.icon:SetBackdropColor(0.6, 0, 0)
		f.text:SetText("MISSING")
		f.text:SetTextColor(0.6, 0, 0)
		f.imbue:SetText("MH")
		f.imbuebg:Show()
		f:Show()
		i=i+1;
	end
	if hasOffHandEnchant and offHandCharges == 0 then
		local name, texture = self:GetWeaponImbueNameTexture(GetInventorySlotInfo("SecondaryHandSlot"))
		local text = self:GetWeaponImbueDuration(offHandExpiration)
		if not self["buff"..i] then
			self:CreateFrame(i)
		end
		local f = self["buff"..i]
		self:DebugMessage("Weapon imbue: "..name, 2)
		self:DebugMessage("Using "..f:GetName().." for "..name, 1)
		f.slot = GetInventorySlotInfo("SecondaryHandSlot")
		f.name = nil
		f.icon:SetBackdrop({bgFile = texture, edgeFile = false, tile=true, tileSize = 31, edgeSize = 0, insets = { left=0, right=0, top=0, bottom=0}})
		f.icon:SetBackdropColor(1, 1, 1)
		f.text:SetText(text)
		f.text:SetTextColor(1, 1, 1)
		f.imbue:SetText("OH")
		f.imbuebg:Show()
		f:Show()
		i=i+1;
	else
		local texture = self:GetMissingTexture((select(2, UnitClass("player"))), true)
		self:DebugMessage("Using "..f:GetName().." for missing buff with texture "..texture, 1)
		if not self["buff"..i] then
			self:CreateFrame(i)
		end
		local f = self["buff"..i]
		f.name = nil
		f.slot = nil
		f.icon:SetBackdrop({bgFile = texture, edgeFile = false, tile = true, tileSize = 31, edgeSize = 0, insets = { left = 0, right = 0, top = 0, bottom = 0}})
		f.icon:SetBackdropColor(0.6, 0, 0)
		f.text:SetText("MISSING")
		f.text:SetTextColor(0.6, 0, 0)
		f.imbue:SetText("OH")
		f.imbuebg:Show()
		f:Show()
		i=i+1;
	end
end

function BuffTheTree:PLAYER_REGEN_DISABLED()
	if not self.opt.incombat then
		self.buff1:Hide()
	end
end

function BuffTheTree:PLAYER_REGEN_ENABLED()
	self.buff1:Show()
end

function BuffTheTree:PARTY_MEMBERS_CHANGED()
	self.buff1[self:IsInRaid() and "Show" or "Hide"](self.buff1)
end

function BuffTheTree:OnInitialize()
	local openOptions = function()
		LibStub("AceConfigDialog-3.0"):Open("BuffTheTree")
	end
	self:RegisterChatCommand("btt", openOptions)
	self.db = LibStub("AceDB-3.0"):New("BuffTheTreeDB", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileEnable")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileEnable")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileEnable")
	self.options = {type = 'group', args = {}}
	self.options.args = self:GetOptions()
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profiles.order = -1,
	LibStub("AceConfig-3.0"):RegisterOptionsTable("BuffTheTree", self.options)
	self.opt = self.db.profile
	self.enabled = false
	self.frames = {}
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "UNIT_AURA")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "UNIT_AURA")
end
